@@ -23,57 +23,49 @@
  * Token filter, for converting a text into a sequence of tokens.
  */
 
-/** ID code for a missing type. */
-#define CORPUS_FILTER_NONE     (-1)
-
-/** ID code for a type that has been ignored. */
-#define CORPUS_FILTER_IGNORED  (-2)
-
-/** ID code for a type that has been dropped. */
-#define CORPUS_FILTER_DROPPED  (-3)
+/** The default word connector for compound tokens */
+#define CORPUS_FILTER_CONNECTOR '_'
 
 /**
  * Filter type, for specifying which word classes to drop.
  */
 enum corpus_filter_type {
-	CORPUS_FILTER_KEEP_ALL = 0,		/**< keep all word types */
-	CORPUS_FILTER_IGNORE_SPACE = (1 << 0),	/**< ignore white space */
-	CORPUS_FILTER_DROP_LETTER  = (1 << 1),	/**< drop letter words */
-	CORPUS_FILTER_DROP_NUMBER  = (1 << 2),	/**< drop number words */
-	CORPUS_FILTER_DROP_PUNCT   = (1 << 3),	/**< drop punctuation words */
-	CORPUS_FILTER_DROP_SYMBOL  = (1 << 4),	/**< drop symbol words */
-	CORPUS_FILTER_IGNORE_OTHER = (1 << 5)	/**< ignore other words */
+	CORPUS_FILTER_KEEP_ALL      = 0,	/**< keep all word types */
+	CORPUS_FILTER_DROP_LETTER   = (1 << 0),	/**< drop letter words */
+	CORPUS_FILTER_DROP_NUMBER   = (1 << 1),	/**< drop number words */
+	CORPUS_FILTER_DROP_PUNCT    = (1 << 2),	/**< drop punctuation words */
+	CORPUS_FILTER_DROP_SYMBOL   = (1 << 3),	/**< drop symbol words */
 };
 
 /**
- * Filter scanning options.
+ * Text filter type properties.
  */
-enum corpus_filter_scan {
-	CORPUS_FILTER_SCAN_TOKENS = 0,	/**< scan a token sequence */
-	CORPUS_FILTER_SCAN_TYPES = 1	/**< scan a type sequence */
+struct corpus_filter_prop {
+	int stem;	/**< stem ID */
+	int unspace;	/**< version with spaces removed */
+
+	int has_stem;	/**< whether the stem has been computed */
+	int has_unspace; /**< whether the unspaced version has
+			    been computed */
+	int drop;	/**< whether to drop the type */
 };
 
 /**
  * Text filter.
  */
 struct corpus_filter {
-	struct corpus_symtab symtab;	/**< symbol table;
-					  we refer to symbol table types as
-					  "symbols" to distinguish them from
-					  filter types */
+	struct corpus_symtab symtab;	/**< symbol table */
+	struct corpus_render render;	/**< type renderer */
 	struct corpus_tree combine;	/**< word sequences to combine */
 	int *combine_rules;		/**< properties for nodes in the
 					  combine tree */
-	int *type_ids;			/**< type IDs for the symbols */
-	int *symbol_ids;		/**< symbol IDs for the types */
-	int ntype;			/**< number of types */
-	int ntype_max;			/**< maximum number of types before
-						requiring reallocation */
+	struct corpus_stem stemmer;	/**< stemmer */
+	int has_stemmer;		/**< whether stemmer is in use */
+	struct corpus_filter_prop *props;/**< type properties */
 	struct corpus_wordscan scan;	/**< current word scan */
 	int flags;			/**< filter flags */
+	uint32_t connector;		/**< word connector */
 	int has_scan;			/**< whether a scan is in progress */
-	int scan_type;			/**< #corpus_filter_scan value
-					  indicating scan type */
 	struct corpus_text current;	/**< current token */
 	int type_id;			/**< current type ID */
 	int error;			/**< last error code */
@@ -83,15 +75,19 @@ struct corpus_filter {
  * Initialize a text filter.
  *
  * \param f the filter
- * \param symbol_kind flags for the symbol table indicating the symbol type kind
- * \param stemmer the stemming algorithm name
  * \param flags a big mask of #corpus_filter_type values indicating filter
  * 	options
+ * \param type_kind flags for the symbol table indicating the type kind
+ * \param connector the word connector to join space-separated
+ *   words in the combination rule, specified as a UTF-32 code
+ * \param stemmer the stemming function
+ * \param context the stemmer context
  *
  * \returns 0 on success
  */
-int corpus_filter_init(struct corpus_filter *f, int symbol_kind,
-		       const char *stemmer, int flags);
+int corpus_filter_init(struct corpus_filter *f, int flags, int type_kind,
+		       uint32_t connector, corpus_stem_func stemmer,
+		       void *context);
 
 /**
  * Release a text filter's resources.
@@ -115,12 +111,25 @@ int corpus_filter_stem_except(struct corpus_filter *f,
  * Add a combination rule to to filter.
  *
  * \param f the filter
- * \param type the type to build a combination rule from
+ * \param tokens the tokens to build a combination rule from
  *
  * \returns 0 on success
  */
 int corpus_filter_combine(struct corpus_filter *f,
-			  const struct corpus_text *type);
+			  const struct corpus_text *tokens);
+
+/**
+ * Add a type to a filter table if it does not already exist there, and
+ * get the id of the type in the filter.
+ *
+ * \param f the filter
+ * \param typ the type
+ * \param idptr a pointer to store the type id, or NULL
+ *
+ * \returns 0 on success
+ */
+int corpus_filter_add_type(struct corpus_filter *f,
+			   const struct corpus_text *typ, int *idptr);
 
 /**
  * Add a type to the filter's drop list.
@@ -151,13 +160,11 @@ int corpus_filter_drop_except(struct corpus_filter *f,
  *
  * \param f the filter
  * \param text the text
- * \param type a #corpus_filter_scan value indicating the scan type
-
  *
  * \returns 0 on success
  */
 int corpus_filter_start(struct corpus_filter *f,
-			const struct corpus_text *text, int type);
+			const struct corpus_text *text);
 
 /**
  * Advance a text to the next type.
@@ -169,15 +176,4 @@ int corpus_filter_start(struct corpus_filter *f,
  */
 int corpus_filter_advance(struct corpus_filter *f);
 
-/**
- * Get the text corresponding to a given type id.
- *
- * \param f the filter
- * \param id the type id
- *
- * \returns the text for that type
- */
-const struct corpus_text *corpus_filter_type(const struct corpus_filter *f,
-					     int id);
-
 #endif /* CORPUS_FILTER_H */
