@@ -30,6 +30,7 @@
 #include "text.h"
 #include "textset.h"
 #include "tree.h"
+#include "stem.h"
 #include "typemap.h"
 #include "symtab.h"
 #include "wordscan.h"
@@ -93,7 +94,7 @@ static int get_arg(const struct string_arg options[], const char *name)
 
 void usage_ngrams(void)
 {
-	const char **stems = corpus_stemmer_names();
+	const char **stems = corpus_stem_snowball_names();
 	const char **stops = corpus_stopword_names();
 	int i;
 
@@ -112,7 +113,6 @@ Options:\n\
 \t-o <path>\tSaves output at the given path.\n\
 \t-s <stemmer>\tStems tokens with the given algorithm.\n\
 \t-t <stopwords>\tDrops words from the given stop word list.\n\
-\t-z\t\tKeeps white space tokens.\n\
 ", PROGRAM_NAME);
 	printf("\nCharacter Maps:\n");
 	for (i = 0; char_maps[i].name != NULL; i++) {
@@ -166,6 +166,7 @@ Options:\n\
 int main_ngrams(int argc, char * const argv[])
 {
 	struct corpus_filter filter;
+	struct corpus_stem_snowball snowball;
 	struct corpus_data data, val;
 	struct corpus_text name, text, word;
 	struct corpus_schema schema;
@@ -183,7 +184,7 @@ int main_ngrams(int argc, char * const argv[])
 	int ch, err, i, name_id, type_id, ncomb;
 	int count;
 
-	filter_flags = CORPUS_FILTER_IGNORE_SPACE | CORPUS_FILTER_IGNORE_OTHER;
+	filter_flags = CORPUS_FILTER_KEEP_ALL;
 	type_flags = (CORPUS_TYPE_MAPCASE | CORPUS_TYPE_MAPCOMPAT
 			| CORPUS_TYPE_MAPQUOTE | CORPUS_TYPE_RMDI);
 
@@ -191,7 +192,7 @@ int main_ngrams(int argc, char * const argv[])
 	length = 1;
 	ncomb = 0;
 
-	while ((ch = getopt(argc, argv, "c:d:f:k:n:o:s:t:z")) != -1) {
+	while ((ch = getopt(argc, argv, "c:d:f:k:n:o:s:t:")) != -1) {
 		switch (ch) {
 		case 'c':
 			if (ncomb == COMBINE_MAX) {
@@ -247,9 +248,6 @@ int main_ngrams(int argc, char * const argv[])
 				return EXIT_FAILURE;
 			}
 			break;
-		case 'z':
-			filter_flags &= ~CORPUS_FILTER_IGNORE_SPACE;
-			break;
 		default:
 			usage_ngrams();
 			return EXIT_FAILURE;
@@ -295,9 +293,21 @@ int main_ngrams(int argc, char * const argv[])
 		goto error_schema;
 	}
 
-	if ((err = corpus_filter_init(&filter, type_flags, stemmer,
-				      filter_flags))) {
-		goto error_filter;
+	if (stemmer) {
+		if ((err = corpus_stem_snowball_init(&snowball, stemmer))) {
+			goto error_snowball;
+		}
+		if ((err = corpus_filter_init(&filter, filter_flags,
+					      type_flags, '_',
+					      corpus_stem_snowball,
+					      &snowball))) {
+			goto error_filter;
+		}
+	} else {
+		if ((err = corpus_filter_init(&filter, filter_flags,
+					      type_flags, '_', NULL, NULL))) {
+			goto error_filter;
+		}
 	}
 
 	if (stopwords) {
@@ -378,8 +388,7 @@ int main_ngrams(int argc, char * const argv[])
 			continue;
 		}
 
-		if ((err = corpus_filter_start(&filter, &text,
-					       CORPUS_FILTER_SCAN_TOKENS))) {
+		if ((err = corpus_filter_start(&filter, &text))) {
 			goto error;
 		}
 
@@ -389,7 +398,7 @@ int main_ngrams(int argc, char * const argv[])
 
 		while (corpus_filter_advance(&filter)) {
 			type_id = filter.type_id;
-			if (type_id == CORPUS_FILTER_IGNORED) {
+			if (type_id == CORPUS_TYPE_NONE) {
 				continue;
 			} else if (type_id < 0) {
 				if ((err = corpus_ngram_break(&ngram))) {
@@ -422,6 +431,10 @@ error_combine:
 error_stopwords:
 	corpus_filter_destroy(&filter);
 error_filter:
+	if (stemmer) {
+		corpus_stem_snowball_destroy(&snowball);
+	}
+error_snowball:
 	corpus_schema_destroy(&schema);
 error_schema:
 	corpus_render_destroy(&render);
