@@ -33,6 +33,7 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(as_list_json, 1),
 	CALLDEF(as_logical_json, 1),
 	CALLDEF(as_text_character, 2),
+	CALLDEF(as_text_filter_connector, 1),
 	CALLDEF(as_text_json, 2),
 	CALLDEF(dim_json, 1),
 	CALLDEF(format_text, 9),
@@ -48,6 +49,7 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(print_json, 1),
 	CALLDEF(read_ndjson, 2),
 	CALLDEF(simplify_json, 1),
+	CALLDEF(stem_snowball, 2),
 	CALLDEF(stopwords, 1),
 	CALLDEF(subscript_json, 2),
 	CALLDEF(subset_json, 3),
@@ -56,7 +58,6 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(text_c, 3),
 	CALLDEF(text_count, 2),
 	CALLDEF(text_detect, 2),
-	CALLDEF(text_length, 1),
 	CALLDEF(text_locate, 2),
 	CALLDEF(text_match, 2),
 	CALLDEF(text_nsentence, 1),
